//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// utility functions for OpenGL game
//
// by Christopher Rasmussen
//
// April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "game_utils.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// MONO characters are all same width: 104.76 pixels

void draw_string(float x, float y, float char_width, char *text, float r, float g, float b, bool mono)
{
  char *p;
   
  glPushMatrix();
  glColor3f(r, g, b);
  glTranslatef(x, y, 0);
  glScalef(char_width / 104.76, char_width / 104.76, 1);
  for (p = text; *p; p++) {

    // characters are all the same widths -- not so pretty, but easier to predict space taken
    
    if (mono)
      glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, *p);
    
    // proportional: looks better, but characters are different widths
    
    else
      glutStrokeCharacter(GLUT_STROKE_ROMAN, *p);
  }

  glPopMatrix();
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// set seed based on time so no two games are alike

void init_random()
{
  struct timeval tp;

  gettimeofday(&tp, NULL);
  srand48(tp.tv_sec);
}

//----------------------------------------------------------------------------

// random float in range [lower, upper]

double uniform_random(double lower, double upper)
{
  double result;
  double range_size;

  range_size = upper - lower;
  result = range_size * drand48();
  result += lower;

  return result;
}

//----------------------------------------------------------------------------

// random int in range [lower, upper]

int uniform_random_int(int lower, int upper)
{
  return (int) round(uniform_random(lower, upper));
}

//----------------------------------------------------------------------------

// set OpenGL states, variables

void init_opengl(string window_title, int window_w, int window_h) 
{
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
  glutInitWindowSize(window_w, window_h);
  glutCreateWindow(window_title.c_str());

  // background color (white -- set first three numbers to 1.0 for black)

  glClearColor(1.0, 1.0, 1.0, 0.0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluOrtho2D(0, window_w, 0, window_h);

  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
