//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// "Game! The Musical"
//
// by Christopher Rasmussen
//
// CISC 181, spring 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "game_utils.hh"
#include "Sprite.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

void randomize_cards();

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// struct/class definitions

struct Card
{
  float x, y;
  int row, col;
};

//----------------------------------------------------------------------------

// constants

const int MAX_STRLEN         = 512;

const int TEXTURE_CARDS      = 10;  // any set of unique positive ints will work here
const int TEXTURE_BACKGROUND = 20;
const int TEXTURE_DOLLAR     = 30;

const float CARD_W = 59.25;
const float CARD_H = 92.25;

const int NUM_SUITES = 4;
const int NUM_CARDS_PER_SUITE = 13;

const int FACEDOWN_ROW = 4;
const int FACEDOWN_COL = 2;

const int NUM_CARDS = 10;

const int SCORE_BAND_HEIGHT = 25;

const int WINDOW_WIDTH = 400;
const int WINDOW_HEIGHT = 400;

const float DELTA_THETA = 1.0;
const float FULL_REVOLUTION = 360.0;

//----------------------------------------------------------------------------

// globals

MultiSprite *card_sprite;
Sprite *background_sprite;
Sprite *dollar1_sprite;
Sprite *dollar2_sprite;

// C string for drawing

char *score_string;

// random cards

Card *card_array;

// position of moveable card

int x_card = 50;
int y_card = 50;
bool card_grabbed = false;

// how to draw the money

int x_dollar1 = 100;
int y_dollar1 = 300;
float theta_dollar1 = 0.0;
bool dollar_spinning = false;

int x_dollar2 = 100;
int y_dollar2 = 200;
float theta_dollar2 = 0.0;

// setting this to false makes textures disappear, but shows sprite dimensions/locations

bool textureFlag = true;

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// has a key been released?

void keyboardup(unsigned char key, int x, int y)
{
  // slide special card back and forth

  if (key == 'a')
    x_card--;
  else if (key == 'd') 
    x_card++;

  // redistribute card locations

  else if (key == 'r') 
    randomize_cards();
  
  // toggle texture display

  else if (key == 't') 
    textureFlag = !textureFlag;

  // toggle spinning dollar

  else if (key == 's') 
    dollar_spinning = !dollar_spinning;

  // end game

  else if (key == 'q')  
    exit(1);

  // redraw now

  glutPostRedisplay();
}

//----------------------------------------------------------------------------

// has a key been pressed _down_?

void keyboard(unsigned char key, int x, int y)
{

}

//----------------------------------------------------------------------------

// has a mouse click event occurred?

void mouse(int button, int state, int x, int y)
{
  if (button == GLUT_LEFT_BUTTON) {

    // are we positioned inside the card sprite?

    if (x >= x_card - CARD_W/2 && x < x_card + CARD_W/2 && 
	(WINDOW_HEIGHT - y) >= y_card - CARD_H/2 && (WINDOW_HEIGHT - y) < y_card + CARD_H/2) {

      // picking it up?

      if (state == GLUT_DOWN) {
	card_grabbed = true;
	x_card = x;
	y_card = WINDOW_HEIGHT - y;
      }

      // or letting go?

      else if (state == GLUT_UP)
	card_grabbed = false;

      // redraw

      glutPostRedisplay();    
    }
  }
}

//----------------------------------------------------------------------------

// mouse moved with button down

void motion(int x, int y)
{
  // drag card around

  if (card_grabbed) {
    x_card = x;
    y_card = WINDOW_HEIGHT - y;

    // redraw

    glutPostRedisplay();
  }
}

//----------------------------------------------------------------------------

// no other events happening...

void idle()
{
  // spin that dollar

  if (dollar_spinning) {

    theta_dollar1 += DELTA_THETA;
    if (theta_dollar1 > FULL_REVOLUTION)
      theta_dollar1 -= FULL_REVOLUTION;
  }

  // redraw

  glutPostRedisplay();
}

//----------------------------------------------------------------------------

// sample shape-drawing function

void draw_line(float x1, float y1, float x2, float y2, float thickness, float r, float g, float b)
{
  // change pen color, size

  glColor3f(r, g, b);
  glLineWidth(thickness);

  glBegin(GL_LINES);
    glVertex2f(x1, y1);
    glVertex2f(x2, y2);
  glEnd();

  // restore pen state

  glColor3f(1,1,1);
  glLineWidth(1);
}

//----------------------------------------------------------------------------

void display() 
{
  // wipe background (don't really need this if drawing a background image)

  glClear(GL_COLOR_BUFFER_BIT);

  // switch into texture mode

  if (textureFlag) {
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
  }

  // draw background

  background_sprite->draw(WINDOW_WIDTH/2, WINDOW_HEIGHT/2, WINDOW_WIDTH, WINDOW_HEIGHT, 0);

  // draw a bunch of random cards

  for (int i = 0; i < NUM_CARDS; i++) {
    card_sprite->draw(card_array[i].col, card_array[i].row, 
		      card_array[i].x, card_array[i].y, 
		      CARD_W, CARD_H, 0, 
		      .9, .9, .9);
  }

  // draw money

  dollar1_sprite->draw(x_dollar1, y_dollar1, dollar1_sprite->imwidth, dollar1_sprite->imheight, theta_dollar1, 
		       .75, .75, .75);

  dollar2_sprite->draw(x_dollar2, y_dollar2, dollar2_sprite->imwidth, dollar2_sprite->imheight, theta_dollar2, 
		       .75, .75, .75);

  // draw one card that we can control

  card_sprite->draw(FACEDOWN_COL, FACEDOWN_ROW, 
		    x_card, y_card, CARD_W, CARD_H, 0, 
		    .75, 0.0, 0.0);

  // switch out of texture mode

  if (textureFlag) {
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);
  }

  // non-texture drawing starts here

  // sample text
  
  sprintf(score_string, "Game! The Musical");
  draw_string(5, WINDOW_HEIGHT - 20, 12, score_string, 0, 1, 0, false);

  // dividing line

  draw_line(0, WINDOW_HEIGHT - SCORE_BAND_HEIGHT, WINDOW_WIDTH - 1, WINDOW_HEIGHT - SCORE_BAND_HEIGHT, 
	    3, 1.0, 0.0, 0.0);

  // finish up

  glutSwapBuffers();
}

//----------------------------------------------------------------------------

// pick random cards and put them in random places
// also, restore dollar to canonical orientation

void randomize_cards()
{
  for (int i = 0; i < NUM_CARDS; i++) {
    card_array[i].row = uniform_random_int(0, NUM_SUITES - 1);
    card_array[i].col = uniform_random_int(0, NUM_CARDS_PER_SUITE - 1);
    card_array[i].x = uniform_random_int(CARD_W/2, WINDOW_WIDTH - CARD_W/2);
    card_array[i].y = uniform_random_int(CARD_H/2, WINDOW_HEIGHT - CARD_H/2 - SCORE_BAND_HEIGHT);
  }

  theta_dollar1 = 0;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

int main(int argc, char** argv)
{
  // boilerplate

  glutInit(&argc, argv);
  init_opengl("game", WINDOW_WIDTH, WINDOW_HEIGHT); 
  init_random();

  // read in sprite source images

  RGB green = {0, 255, 0};
  RGB none = {-1, -1, -1};

  clear_sprites();                                            // call this once, _before_ adding any info

  add_sprite_info("cards.ppm", TEXTURE_CARDS, green);         // call this for each source image file
  add_sprite_info("background.ppm", TEXTURE_BACKGROUND, none);   
  add_sprite_info("onedollar.ppm", TEXTURE_DOLLAR, none);   

  finalize_sprites();                                         // call this once, _after_ adding all info

  // make actual sprites from source images

  card_sprite = new MultiSprite(13, 5, TEXTURE_CARDS);  
  background_sprite = new Sprite(TEXTURE_BACKGROUND);
  dollar1_sprite = new Sprite(TEXTURE_DOLLAR);
  dollar2_sprite = new Sprite(TEXTURE_DOLLAR);

  // initialize other data structures

  card_array = new Card[NUM_CARDS];
  randomize_cards();

  score_string = (char *) malloc(MAX_STRLEN * sizeof(char));   

  // register callback routines

  glutDisplayFunc(display);
  glutKeyboardFunc(keyboard);
  glutKeyboardUpFunc(keyboardup);
  glutMotionFunc(motion);
  glutMouseFunc(mouse);
  glutIdleFunc(idle);

  // enter event-driven loop

  glutMainLoop(); 		    

  // this will never be reached

  return 1;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
